==> kv5500-0.1.tar.gz: kv5500-0.1/src/kv5500_phonebook.cpp <==
/*****************************************************************************
 *
 *  kv5500_phonebook - read a phonebook from kv5500 like phone.
 *
 * Copyright (c) 2006 Hyo geol, Lee
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 *   3. The name of the author may not be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 *
 ****************************************************************************/

#include <cstdio>
#include <cstring>

#include "kv5500.hpp"
#include "kv5500_phonebook.hpp"

using namespace KV5500;

bool PhonebookScanner::DoScan(struct usb_device* dev)
{
    assert(dev);

    try {

        UsbDev udev(dev);

        if(DescCheck(dev)) {

            if(!WakeUp(udev)) {

                printf("Cannot wake up phone.\n");

                return false;
            }

            if(!RequestFirst(udev))
                return false;

            if(!RequestRest(udev))
                return false;
        }

    } catch(std::exception& e) {

        printf("%s\n", e.what());
    }

    return true;
}

/// buf[2] == 0x01          empty data
/// buf[offset]             start of name, offset maybe 9 or 11
/// buf[offset+34]          start of tel number, offset maybe 9 or 11
///                         each number size is 33. tel number maybe up to 5.
int PhonebookScanner::Recv(UsbDev& udev, unsigned int offset)
{
    char buf[1000];
    const int ret = udev.BulkRead(buf, sizeof(buf), 5000);

    /// assume ret must over 45 bytes. maybe empty data when 0x01
    if(ret <= 45 || buf[2] == 0x01)
        return -1;

    PhonebookData thisData;

    const char* const nameStr = GetString(buf, ret - offset, offset, 33, 0x00);
    thisData.SetName(nameStr);
    delete [] nameStr;

    std::vector<std::string> telVec;
    GetPhonebookTelStr(telVec, buf, offset);
    thisData.SetNumber(telVec);

    phonebook_.push_back(thisData);

    return ret;
}

bool PhonebookScanner::RequestFirst(UsbDev& udev)
{
    /// request first phone book data
    char buf[3] = { 0xfe, 0x46, 0x7e };
    if(udev.BulkWrite(buf, sizeof(buf), 5000) < 0) {

        printf("Got error while send.\n");

        return false;

    } else if(Recv(udev, 9) < 0)
        return false;

    return true;
}

bool PhonebookScanner::RequestRest(UsbDev& udev)
{
    /// request rest phone book data.
    /// I don't know total count yet, so big loop.
    char buf[3] = { 0xfe, 0x47, 0x7e };
    for(int i = 0; i < 65535; ++i) {

        if(udev.BulkWrite(buf, sizeof(buf), 5000) < 0) {

            printf("Got error while send.\n");

            return false;

        } else if(Recv(udev, 11) < 0)
            break;
    }

    return true;
}

void KV5500::GetPhonebookTelStr(std::vector<std::string>& vec, const char* buf, const unsigned int offset)
{
    /// get total telephone number(maybe up to 5),
    const int totalTelNumber = buf[offset + 33];
    for(int i = 0; i < totalTelNumber; ++i) {

        const char* const telStr = GetString(buf, 33, offset + 34 + i * 33, 33, 0x00);
        vec.push_back(telStr);

        delete [] telStr;
    }
}

void KV5500::GetPhonebook(std::vector<PhonebookData>& phonebook)
{
    Usb thisUsb;
    PhonebookScanner pbScanner;

    thisUsb.Scan(pbScanner);

    pbScanner.GetPhonebook(phonebook);
}

==> kv5500-0.2.tar.gz: kv5500-0.2/src/kv5500_phonebook.cpp <==
/*****************************************************************************
 *
 *  kv5500_phonebook - read a phonebook from kv5500 like phone.
 *
 * Copyright (c) 2006 Hyo geol, Lee
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 *   3. The name of the author may not be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 *
 ****************************************************************************/

#include <cstdio>
#include <cstring>

#include "kv5500.hpp"
#include "kv5500_phonebook.hpp"

using namespace KV5500;

bool PhonebookScanner::DoScan(struct usb_device* dev)
{
    assert(dev);

    try {

        UsbDev udev(dev);

        if(DescCheck(dev)) {

            if(!WakeUp(udev)) {

                printf("Cannot wake up phone.\n");

                return false;
            }

            if(!RequestFirst(udev))
                return false;

            if(!RequestRest(udev))
                return false;
        }

    } catch(std::exception& e) {

        printf("%s\n", e.what());
    }

    return true;
}

/// buf[2] == 0x01          empty data
/// buf[offset]             start of name, offset maybe 9 or 11
/// buf[offset+34]          start of tel number, offset maybe 9 or 11
///                         each number size is 33. tel number maybe up to 5.
int PhonebookScanner::Recv(UsbDev& udev, unsigned int offset)
{
    char buf[1000];
    const int ret = udev.BulkRead(buf, sizeof(buf), 5000);

    /// assume ret must over 45 bytes. maybe empty data when 0x01
    if(ret <= 45 || buf[2] == 0x01)
        return -1;

    PhonebookData thisData(GetString(buf, ret - offset, offset, 33, 0x00));

    std::vector<std::string> telVec;
    GetPhonebookTelStr(telVec, buf, offset);
    thisData.SetNumber(telVec);

    phonebook_.push_back(thisData);

    return ret;
}

bool PhonebookScanner::RequestFirst(UsbDev& udev)
{
    /// request first phone book data
    char buf[3] = { 0xfe, 0x46, 0x7e };
    if(udev.BulkWrite(buf, sizeof(buf), 5000) < 0) {

        printf("Got error while send.\n");

        return false;

    } else if(Recv(udev, 9) < 0)
        return false;

    return true;
}

bool PhonebookScanner::RequestRest(UsbDev& udev)
{
    /// request rest phone book data.
    /// I don't know total count yet, so big loop.
    char buf[3] = { 0xfe, 0x47, 0x7e };
    for(int i = 0; i < 65535; ++i) {

        if(udev.BulkWrite(buf, sizeof(buf), 5000) < 0) {

            printf("Got error while send.\n");

            return false;

        } else if(Recv(udev, 11) < 0)
            break;
    }

    return true;
}

void KV5500::GetPhonebookTelStr(std::vector<std::string>& vec, const char* buf, const unsigned int offset)
{
    /// get total telephone number(maybe up to 5),
    const int totalTelNumber = buf[offset + 33];
    for(int i = 0; i < totalTelNumber; ++i) {

        vec.push_back(GetString(buf, 33, offset + 34 + i * 33, 33, 0x00));
    }
}

void KV5500::GetPhonebook(std::vector<PhonebookData>& phonebook)
{
    Usb thisUsb;
    PhonebookScanner pbScanner;

    thisUsb.Scan(pbScanner);

    pbScanner.GetPhonebook(phonebook);
}
